#include <gtk/gtk.h>

GtkWidget *tree;

void populate_tree(void);
void clear_only(GtkWidget*, gpointer);
void clear_with_deselect(GtkWidget*, gpointer);

gboolean show_alpha = TRUE;

int
main(int argc, char *argv[])
{
	GtkWidget *window, *clear_only_button, *clear_deselect_button,
		  *tree_sw, *vbox;

	gtk_init(&argc, &argv);

	/* Main window */
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_name(window, "main window");
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
		GTK_SIGNAL_FUNC(gtk_main_quit), NULL);

	/* Main vbox */
	vbox = gtk_vbox_new(FALSE, 1);
	gtk_container_add(GTK_CONTAINER(window), vbox);
	gtk_widget_show(vbox);

	/* Tree's scroll-window */
	tree_sw = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(tree_sw),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(vbox), tree_sw);
	gtk_widget_show(tree_sw);
	
	/* Tree */
	tree = gtk_tree_new();
	gtk_tree_set_selection_mode(GTK_TREE(tree), GTK_SELECTION_SINGLE);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(tree_sw), tree);
	gtk_widget_show(tree);

	populate_tree();

	/* Clear-only button */
	clear_only_button = gtk_button_new_with_label("Clear tree with gtk_tree_clear_items()");
	gtk_signal_connect(GTK_OBJECT(clear_only_button), "clicked",
		GTK_SIGNAL_FUNC(clear_only), NULL);
	gtk_container_add(GTK_CONTAINER(vbox), clear_only_button);
	gtk_widget_show(clear_only_button);

	/* Clear with deselect button */
	clear_deselect_button = gtk_button_new_with_label("Clear tree with extra deselect");
	gtk_signal_connect(GTK_OBJECT(clear_deselect_button), "clicked",
		GTK_SIGNAL_FUNC(clear_with_deselect), NULL);
	gtk_container_add(GTK_CONTAINER(vbox), clear_deselect_button);
	gtk_widget_show(clear_deselect_button);

	gtk_widget_show(window);

	gtk_main();

	exit(0);
}

void
clear_only(GtkWidget *w, gpointer data)
{
	gtk_tree_clear_items(GTK_TREE(tree), 0, -1);
	populate_tree();
}

void
clear_with_deselect(GtkWidget *w, gpointer data)
{
	GList *selection;
	GtkWidget *tmp_item;

	if (GTK_TREE(tree)->root_tree) {
		selection = GTK_TREE(tree)->root_tree->selection;
		while (selection) {
			tmp_item = selection->data;
			gtk_tree_item_deselect(GTK_TREE_ITEM(tmp_item));
			gtk_widget_unref(tmp_item);
			selection = selection->next;
		}
		g_list_free(GTK_TREE(tree)->root_tree->selection);
		GTK_TREE(tree)->root_tree->selection = NULL;
	}

	gtk_tree_clear_items(GTK_TREE(tree), 0, -1);
	populate_tree();
}

void
populate_tree(void)
{
	GtkWidget	*ti, *subtree;

	if (show_alpha == TRUE) {
		ti = gtk_tree_item_new_with_label("Label A");
		gtk_tree_append(GTK_TREE(tree), ti);
		gtk_widget_show(ti);

		subtree = gtk_tree_new();
		gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), subtree);
		ti = gtk_tree_item_new_with_label("Sub-Label A");
		gtk_tree_append(GTK_TREE(subtree), ti);
		gtk_widget_show(ti);

		ti = gtk_tree_item_new_with_label("Label B");
		gtk_tree_append(GTK_TREE(tree), ti);
		gtk_widget_show(ti);

		ti = gtk_tree_item_new_with_label("Label C");
		gtk_tree_append(GTK_TREE(tree), ti);
		gtk_widget_show(ti);
	}
	else {
		ti = gtk_tree_item_new_with_label("Label 1");
		gtk_tree_append(GTK_TREE(tree), ti);
		gtk_widget_show(ti);

		subtree = gtk_tree_new();
		gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), subtree);
		ti = gtk_tree_item_new_with_label("Sub-Label 1");
		gtk_tree_append(GTK_TREE(subtree), ti);
		gtk_widget_show(ti);

		ti = gtk_tree_item_new_with_label("Label 2");
		gtk_tree_append(GTK_TREE(tree), ti);
		gtk_widget_show(ti);

		ti = gtk_tree_item_new_with_label("Label 3");
		gtk_tree_append(GTK_TREE(tree), ti);
		gtk_widget_show(ti);
	}

	show_alpha = ! show_alpha;
}
